@@ -1,4 +1,4 @@
-# $Id: Setpwd.pm 37763 2012-09-06 14:41:46Z anton $
+# $Id: Setpwd.pm 37883 2012-09-27 11:28:11Z anton $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/pwdmodifier/keymaster/lib/Setpwd.pm $
 package Setpwd;
 use strict;
@@ -38,7 +38,7 @@ sub new {
   $self->allows(
 	  [ qw (challenge time oracle_testanr mailalso2 skiporacle dryrun untouchables newPwd
             silentsshclients silentclients sync-passwords reset-password contact language
-		     syncOnlyIfNeeded verifyPasswordSet adminMailAddress ldapRetries ) ]
+		     syncOnlyIfNeeded verifyPasswordSet adminMailAddress ldapRetries suspend ) ]
 	  );
 
   $self->allows({'tildeusername' => ''});
@@ -194,11 +194,13 @@ sub fetchCurrentLdapPassword {
 	die "No entry found for ldaphost: $ldaphost" unless ($entry);
 
 	my $rcryptpwd =	$entry->get_value('rcryptpassword');
-	die "NO RCRYPTPWD FOUND for: $self->{fields}->{anr}" unless defined($rcryptpwd);
-	die "userError: Account DISABLED, invalid rcrypt found" unless $rcryptpwd=~s/^\{rcrypt\}//;
+	die "NO RCRYPTPWD FOUND for: $self->{fields}->{anr}" unless $rcryptpwd;
+	die "userError: Account DISABLED, invalid rcrypt found" unless $rcryptpwd=~s/\{rcrypt\}//;
+	$rcryptpwd =~ s/^suspended\://g;
 
 	my $decrypted = Rcrypt::decrypt($rcryptpwd,$self->{rcryptkey});
 	if ($decrypted) {
+		warn "decrypted: $decrypted";
  		$self->debug("Current ldapPassword retreived for: $anr");
 	} else {
 		warn "Error: Current ldapPassword NOT retreived for: $anr";
@@ -220,8 +222,8 @@ sub synchronisePassword {
 
 	$ret->{oracle} = $self->fetchCurrentSQLPassword();
 	($ret->{rcrypt}, $ret->{ldap}) = $self->fetchCurrentLdapPassword();
-
 	$self->{fields}->{newPwd} = $ret->{$passwordLeader};
+
 	if ($self->{fields}->{syncOnlyIfNeeded}){
 		if(exists ($ret->{rcrypt}) and exists ($ret->{oracle}) and exists ($ret->{ldap})){
 			if(($ret->{oracle} eq $ret->{ldap}) and ($ret->{oracle} eq $ret->{rcrypt})) {
@@ -233,6 +235,7 @@ sub synchronisePassword {
 
 	# If the passwordLeader did not provide a password get the current decrypted password from ldap or fail.
 	unless ($self->{fields}->{newPwd}){
+		warn "no password yet";
 		if ($ret->{ldap}) {
 			$self->{fields}->{newPwd} = $ret->{ldap}
 		} else {
@@ -486,8 +489,10 @@ sub setPassword {
 	$self->{fields}->{dryrun} = 'no' unless (exists ($self->{fields}->{dryrun}));
 	$self->debug( "Dryrun state: " .$self->{fields}->{dryrun});
 	my $newfields;
+	my $newPwd = $self->{fields}->{newPwd};
 
-	$newfields->{rcryptpassword} = $self->rcrypt($self->{fields}->{newPwd}, $self->{rcryptkey});
+	$newfields->{rcryptpassword} = undef;
+	$newfields->{rcryptpassword} = $self->rcrypt($newPwd, $self->{rcryptkey}) if $newPwd;
 	$newfields->{datepasswdchanged} = time;
 
 	#prepare oraclechanges, no commit yet
@@ -531,6 +536,16 @@ sub setPassword {
 						warn ("\tignoring invalid attrib for host $server: $attrib");
 						next;
 					}
+#					next if  $attrib eq 'rcryptpassword' and $self->{fields}->{suspend};
+					if ($attrib eq 'rcryptpassword' and $self->{fields}->{suspend}) {
+						my $rcryptpassword = $entry->get_value('rcryptpassword');
+						unless ($rcryptpassword =~ /^suspendend:/) {
+							my $suspendedRcryptpassword = "suspended:"."$rcryptpassword";
+							$entry->replace($attrib => $suspendedRcryptpassword);
+						}
+						next;
+					}
+
 					if ($newfields->{$attrib}) {
 						$self->debug("\tchange: $attrib into $newfields->{$attrib}");
 						$entry->replace($attrib => $newfields->{$attrib});
